import bpy

class KTC_VisibilityCheckItem(bpy.types.PropertyGroup):
    name: bpy.props.StringProperty(name="Object Name")
    is_collection: bpy.props.BoolProperty(name="Is Collection", default=False)

class KTC_VisibilityCheckProperties(bpy.types.PropertyGroup):
    visibility_list: bpy.props.CollectionProperty(type=KTC_VisibilityCheckItem)
    active_index: bpy.props.IntProperty(update=lambda self, context: self.update_active_index(context))
    show_collections: bpy.props.BoolProperty(
        name="Show Collections",
        description="Show collections with mismatched visibility",
        default=False
    )

    def update_active_index(self, context):
        if self.active_index >= 0 and self.active_index < len(self.visibility_list):
            item = self.visibility_list[self.active_index]
            if item.is_collection:
                collection = bpy.data.collections.get(item.name)
                if collection:
                    bpy.ops.object.select_all(action='DESELECT')
                    for obj in collection.objects:
                        obj.select_set(True)
                    context.view_layer.objects.active = collection.objects[0] if collection.objects else None
            else:
                obj = context.scene.objects.get(item.name)
                if obj:
                    bpy.ops.object.select_all(action='DESELECT')
                    obj.select_set(True)
                    context.view_layer.objects.active = obj

def is_collection_excluded(layer_collection, obj_collection):
    if layer_collection.collection == obj_collection:
        return layer_collection.exclude
    for child in layer_collection.children:
        if is_collection_excluded(child, obj_collection):
            return True
    return False

class KTC_OBJECT_OT_check_visibility(bpy.types.Operator):
    bl_idname = "object.check_visibility"
    bl_label = "Check Visibility"
    bl_description = "Check the visibility of objects and collections in Viewport and Render"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        visibility_check = context.scene.visibility_check
        visibility_check.visibility_list.clear()

        # Check objects
        for obj in context.scene.objects:
            if any(is_collection_excluded(layer, obj.users_collection[0]) for layer in context.view_layer.layer_collection.children):
                continue
            if obj.hide_viewport != obj.hide_render:
                item = visibility_check.visibility_list.add()
                item.name = obj.name
        
        # Check collections if the option is enabled
        if visibility_check.show_collections:
            for collection in bpy.data.collections:
                if any(is_collection_excluded(layer, collection) for layer in context.view_layer.layer_collection.children):
                    continue
                if collection.hide_viewport != collection.hide_render:
                    item = visibility_check.visibility_list.add()
                    item.name = collection.name
                    item.is_collection = True
        
        if not visibility_check.visibility_list:
            item = visibility_check.visibility_list.add()
            item.name = "No mismatched objects or collections found"
        
        return {'FINISHED'}

class KTC_OBJECT_OT_equalize_to_render(bpy.types.Operator):
    bl_idname = "object.equalize_to_render"
    bl_label = "Equalize to Render"
    bl_description = "Set Viewport visibility to match Render visibility for all objects in the list"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        visibility_check = context.scene.visibility_check
        
        for item in visibility_check.visibility_list:
            if item.name == "No mismatched objects or collections found":
                continue
            if item.is_collection:
                collection = bpy.data.collections.get(item.name)
                if collection:
                    collection.hide_viewport = collection.hide_render
            else:
                obj = context.scene.objects[item.name]
                obj.hide_viewport = obj.hide_render
        
        return {'FINISHED'}

class KTC_OBJECT_OT_equalize_to_viewport(bpy.types.Operator):
    bl_idname = "object.equalize_to_viewport"
    bl_label = "Equalize to Viewport"
    bl_description = "Set Render visibility to match Viewport visibility for all objects in the list"
    bl_options = {'REGISTER', 'UNDO'}

    def execute(self, context):
        visibility_check = context.scene.visibility_check
        
        for item in visibility_check.visibility_list:
            if item.name == "No mismatched objects or collections found":
                continue
            if item.is_collection:
                collection = bpy.data.collections.get(item.name)
                if collection:
                    collection.hide_render = collection.hide_viewport
            else:
                obj = context.scene.objects[item.name]
                obj.hide_render = obj.hide_viewport
        
        return {'FINISHED'}

class KTC_OBJECT_OT_select_object(bpy.types.Operator):
    bl_idname = "object.select_object"
    bl_label = "Select Object"
    bl_description = "Select this object in the viewport"
    bl_options = {'REGISTER', 'UNDO'}
    
    object_name: bpy.props.StringProperty()

    def execute(self, context):
        obj = context.scene.objects.get(self.object_name)
        if obj:
            bpy.ops.object.select_all(action='DESELECT')
            obj.select_set(True)
            context.view_layer.objects.active = obj
        return {'FINISHED'}

class KTC_UI_UL_visibility_check_list(bpy.types.UIList):
    def draw_item(self, context, layout, data, item, icon, active_data, active_propname, index):
        if item.name == "No mismatched objects or collections found":
            layout.label(text=item.name)
        else:
            if item.is_collection:
                collection = bpy.data.collections.get(item.name)
                if collection:
                    row = layout.row(align=True)
                    row.label(text=collection.name, icon='OUTLINER_COLLECTION')
                    row.prop(collection, "hide_viewport", text="", emboss=False)
                    row.separator()
                    row.prop(collection, "hide_render", text="", emboss=False)
                    row.separator()
                    row.operator("object.select_object", text="", icon='RESTRICT_SELECT_OFF', emboss=False).object_name = collection.name
            else:
                obj = context.scene.objects[item.name]
                row = layout.row(align=True)
                icon = self.get_icon(obj.type)
                row.label(text=obj.name, icon=icon)
                row.prop(obj, "hide_viewport", text="", emboss=False)
                row.separator()
                row.prop(obj, "hide_render", text="", emboss=False)
                row.separator()
                row.operator("object.select_object", text="", icon='RESTRICT_SELECT_OFF', emboss=False).object_name = obj.name

    def get_icon(self, obj_type):
        icons = {
            'MESH': 'MESH_CUBE',
            'CURVE': 'CURVE_BEZCURVE',
            'SURFACE': 'SURFACE_NCURVE',
            'META': 'META_BALL',
            'FONT': 'FONT_DATA',
            'ARMATURE': 'ARMATURE_DATA',
            'LATTICE': 'LATTICE_DATA',
            'EMPTY': 'EMPTY_DATA',
            'LIGHT': 'LIGHT_DATA',
            'SPEAKER': 'SPEAKER',
            'CAMERA': 'CAMERA_DATA',
            # Adicione mais tipos e ícones correspondentes conforme necessário
        }
        return icons.get(obj_type, 'QUESTION')

class KTC_VIEW3D_PT_visibility_check_panel(bpy.types.Panel):
    bl_label = "Visibility Check"
    bl_idname = "KTC_VIEW3D_PT_visibility_check_panel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = 'K-Tools'
    bl_options = {"DEFAULT_CLOSED"}

    def draw_header(self, context):
        layout = self.layout
        layout.label(text="", icon="HIDE_OFF")
    
    def draw(self, context):
        layout = self.layout
        visibility_check = context.scene.visibility_check
        
        layout.operator("object.check_visibility")

        layout.prop(visibility_check, "show_collections", text="Show Collections")

        layout.label(text="Mismatched Objects and Collections:")
        layout.template_list("KTC_UI_UL_visibility_check_list", "", visibility_check, "visibility_list", visibility_check, "active_index")
        
        layout.operator("object.hide_view_clear", text="Unhide All")
        
        row = layout.row(align=True)
        row.operator("object.equalize_to_render", text="Equalize to Render")
        row.operator("object.equalize_to_viewport", text="Equalize to Viewport")

classes = [
    KTC_VisibilityCheckItem,
    KTC_VisibilityCheckProperties,
    KTC_OBJECT_OT_check_visibility,
    KTC_OBJECT_OT_equalize_to_render,
    KTC_OBJECT_OT_equalize_to_viewport,
    KTC_OBJECT_OT_select_object,
    KTC_UI_UL_visibility_check_list,
    KTC_VIEW3D_PT_visibility_check_panel,
]

def register():
    for cls in classes:
        bpy.utils.register_class(cls)
    bpy.types.Scene.visibility_check = bpy.props.PointerProperty(type=KTC_VisibilityCheckProperties)

def unregister():
    for cls in classes:
        bpy.utils.unregister_class(cls)
    del bpy.types.Scene.visibility_check

if __name__ == "__main__":
    register()

